@@ -44,12 +44,12 @@
 #define cstr2num(s) rb_cstr_to_inum(s, 10, 0)
 #define str2num(s) rb_str_to_inum(s, 10, 0)
 
-static const char *abbr_days[] = {
+static const char abbr_days[][4] = {
     "sun", "mon", "tue", "wed",
     "thu", "fri", "sat"
 };
 
-static const char *abbr_months[] = {
+static const char abbr_months[][4] = {
     "jan", "feb", "mar", "apr", "may", "jun",
     "jul", "aug", "sep", "oct", "nov", "dec"
 };
@@ -57,16 +57,21 @@ static const char *abbr_months[] = {
 #define issign(c) ((c) == '-' || (c) == '+')
 #define asp_string() rb_str_new(" ", 1)
 #ifdef TIGHT_PARSER
+#define asuba_string() rb_str_new("\001", 1)
+#define asubb_string() rb_str_new("\002", 1)
 #define asubw_string() rb_str_new("\027", 1)
 #define asubt_string() rb_str_new("\024", 1)
 #endif
 
+#define DECDIGIT "0123456789"
+
 static void
 s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 {
+    VALUE vbuf = 0;
     VALUE c = Qnil;
 
-    if (TYPE(m) != T_STRING)
+    if (!RB_TYPE_P(m, T_STRING))
 	m = f_to_s(m);
 
     if (!NIL_P(y) && !NIL_P(m) && NIL_P(d)) {
@@ -100,7 +105,7 @@ s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 	bp = s;
 	if (issign((unsigned char)*s))
 	    s++;
-	l = strspn(s, "0123456789");
+	l = strspn(s, DECDIGIT);
 	ep = s + l;
 	if (*ep) {
 	    y = d;
@@ -153,23 +158,25 @@ s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 	}
 	if (sign)
 	    c = Qfalse;
-	l = strspn(s, "0123456789");
+	l = strspn(s, DECDIGIT);
 	ep = s + l;
 	if (l > 2)
 	    c = Qfalse;
 	{
 	    char *buf;
 
-	    buf = ALLOCA_N(char, ep - bp + 1);
+	    buf = ALLOCV_N(char, vbuf, ep - bp + 1);
 	    memcpy(buf, bp, ep - bp);
 	    buf[ep - bp] = '\0';
 	    iy = cstr2num(buf);
+	    ALLOCV_END(vbuf);
 	}
-	if (bc)
-	    iy = f_add(f_negate(iy), INT2FIX(1));
 	set_hash("year", iy);
     }
 
+    if (bc)
+	set_hash("_bc", Qtrue);
+
     if (!NIL_P(m)) {
 	const char *s, *bp, *ep;
 	size_t l;
@@ -179,15 +186,16 @@ s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 	while (!isdigit((unsigned char)*s))
 	    s++;
 	bp = s;
-	l = strspn(s, "0123456789");
+	l = strspn(s, DECDIGIT);
 	ep = s + l;
 	{
 	    char *buf;
 
-	    buf = ALLOCA_N(char, ep - bp + 1);
+	    buf = ALLOCV_N(char, vbuf, ep - bp + 1);
 	    memcpy(buf, bp, ep - bp);
 	    buf[ep - bp] = '\0';
 	    im = cstr2num(buf);
+	    ALLOCV_END(vbuf);
 	}
 	set_hash("mon", im);
     }
@@ -201,15 +209,16 @@ s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 	while (!isdigit((unsigned char)*s))
 	    s++;
 	bp = s;
-	l = strspn(s, "0123456789");
+	l = strspn(s, DECDIGIT);
 	ep = s + l;
 	{
 	    char *buf;
 
-	    buf = ALLOCA_N(char, ep - bp + 1);
+	    buf = ALLOCV_N(char, vbuf, ep - bp + 1);
 	    memcpy(buf, bp, ep - bp);
 	    buf[ep - bp] = '\0';
 	    id = cstr2num(buf);
+	    ALLOCV_END(vbuf);
 	}
 	set_hash("mday", id);
     }
@@ -224,10 +233,12 @@ s3e(VALUE hash, VALUE y, VALUE m, VALUE d, int bc)
 #define ABBR_MONTHS "jan|feb|mar|apr|may|jun|jul|aug|sep|oct|nov|dec"
 
 #ifdef TIGHT_PARSER
-#define VALID_DAYS "(?:" DAYS ")\\b" "|(?:tues|wednes|thur|thurs|" ABBR_DAYS ")\\b\\.?"
-#define VALID_MONTHS "(?:" MONTHS ")\\b" "|(?:sept|" ABBR_MONTHS ")\\b\\.?"
-#define DOTLESS_VALID_MONTHS "(?:" MONTHS ")\\b" "|(?:sept|" ABBR_MONTHS ")\\b"
+#define VALID_DAYS "(?:" DAYS ")" "|(?:tues|wednes|thurs|thur|" ABBR_DAYS ")\\.?"
+#define VALID_MONTHS "(?:" MONTHS ")" "|(?:sept|" ABBR_MONTHS ")\\.?"
+#define DOTLESS_VALID_MONTHS "(?:" MONTHS ")" "|(?:sept|" ABBR_MONTHS ")"
 #define BOS "\\A\\s*"
+#define FPA "\\001"
+#define FPB "\\002"
 #define FPW "\\027"
 #define FPT "\\024"
 #define FPW_COM "\\s*(?:" FPW "\\s*,?)?\\s*"
@@ -305,6 +316,16 @@ subx(VALUE str, VALUE rep, VALUE pat, VALUE hash, int (*cb)(VALUE, VALUE))
 }
 
 #ifdef TIGHT_PARSER
+#define SUBA(s,p,c) \
+{ \
+    return subx(s, asuba_string(), p, hash, c); \
+}
+
+#define SUBB(s,p,c) \
+{ \
+    return subx(s, asubb_string(), p, hash, c); \
+}
+
 #define SUBW(s,p,c) \
 { \
     return subx(s, asubw_string(), p, hash, c); \
@@ -316,93 +337,13 @@ subx(VALUE str, VALUE rep, VALUE pat, VALUE hash, int (*cb)(VALUE, VALUE))
 }
 #endif
 
-struct zone {
-    const char *name;
-    int offset;
-};
-
-static struct zone zones_source[] = {
-    {"ut",   0*3600}, {"gmt",  0*3600}, {"est", -5*3600}, {"edt", -4*3600},
-    {"cst", -6*3600}, {"cdt", -5*3600}, {"mst", -7*3600}, {"mdt", -6*3600},
-    {"pst", -8*3600}, {"pdt", -7*3600},
-    {"a",    1*3600}, {"b",    2*3600}, {"c",    3*3600}, {"d",    4*3600},
-    {"e",    5*3600}, {"f",    6*3600}, {"g",    7*3600}, {"h",    8*3600},
-    {"i",    9*3600}, {"k",   10*3600}, {"l",   11*3600}, {"m",   12*3600},
-    {"n",   -1*3600}, {"o",   -2*3600}, {"p",   -3*3600}, {"q",   -4*3600},
-    {"r",   -5*3600}, {"s",   -6*3600}, {"t",   -7*3600}, {"u",   -8*3600},
-    {"v",   -9*3600}, {"w",  -10*3600}, {"x",  -11*3600}, {"y",  -12*3600},
-    {"z",    0*3600},
-
-    {"utc",  0*3600}, {"wet",  0*3600},
-    {"at",  -2*3600}, {"brst",-2*3600}, {"ndt", -(2*3600+1800)},
-    {"art", -3*3600}, {"adt", -3*3600}, {"brt", -3*3600}, {"clst",-3*3600},
-    {"nst", -(3*3600+1800)},
-    {"ast", -4*3600}, {"clt", -4*3600},
-    {"akdt",-8*3600}, {"ydt", -8*3600},
-    {"akst",-9*3600}, {"hadt",-9*3600}, {"hdt", -9*3600}, {"yst", -9*3600},
-    {"ahst",-10*3600},{"cat",-10*3600}, {"hast",-10*3600},{"hst",-10*3600},
-    {"nt",  -11*3600},
-    {"idlw",-12*3600},
-    {"bst",  1*3600}, {"cet",  1*3600}, {"fwt",  1*3600}, {"met",  1*3600},
-    {"mewt", 1*3600}, {"mez",  1*3600}, {"swt",  1*3600}, {"wat",  1*3600},
-    {"west", 1*3600},
-    {"cest", 2*3600}, {"eet",  2*3600}, {"fst",  2*3600}, {"mest", 2*3600},
-    {"mesz", 2*3600}, {"sast", 2*3600}, {"sst",  2*3600},
-    {"bt",   3*3600}, {"eat",  3*3600}, {"eest", 3*3600}, {"msk",  3*3600},
-    {"msd",  4*3600}, {"zp4",  4*3600},
-    {"zp5",  5*3600}, {"ist",  (5*3600+1800)},
-    {"zp6",  6*3600},
-    {"wast", 7*3600},
-    {"cct",  8*3600}, {"sgt",  8*3600}, {"wadt", 8*3600},
-    {"jst",  9*3600}, {"kst",  9*3600},
-    {"east",10*3600}, {"gst", 10*3600},
-    {"eadt",11*3600},
-    {"idle",12*3600}, {"nzst",12*3600}, {"nzt", 12*3600},
-    {"nzdt",13*3600},
-
-    {"afghanistan",             16200}, {"alaskan",                -32400},
-    {"arab",                    10800}, {"arabian",                 14400},
-    {"arabic",                  10800}, {"atlantic",               -14400},
-    {"aus central",             34200}, {"aus eastern",             36000},
-    {"azores",                  -3600}, {"canada central",         -21600},
-    {"cape verde",              -3600}, {"caucasus",                14400},
-    {"cen. australia",          34200}, {"central america",        -21600},
-    {"central asia",            21600}, {"central europe",           3600},
-    {"central european",         3600}, {"central pacific",         39600},
-    {"central",                -21600}, {"china",                   28800},
-    {"dateline",               -43200}, {"e. africa",               10800},
-    {"e. australia",            36000}, {"e. europe",                7200},
-    {"e. south america",       -10800}, {"eastern",                -18000},
-    {"egypt",                    7200}, {"ekaterinburg",            18000},
-    {"fiji",                    43200}, {"fle",                      7200},
-    {"greenland",              -10800}, {"greenwich",                   0},
-    {"gtb",                      7200}, {"hawaiian",               -36000},
-    {"india",                   19800}, {"iran",                    12600},
-    {"jerusalem",                7200}, {"korea",                   32400},
-    {"mexico",                 -21600}, {"mid-atlantic",            -7200},
-    {"mountain",               -25200}, {"myanmar",                 23400},
-    {"n. central asia",         21600}, {"nepal",                   20700},
-    {"new zealand",             43200}, {"newfoundland",           -12600},
-    {"north asia east",         28800}, {"north asia",              25200},
-    {"pacific sa",             -14400}, {"pacific",                -28800},
-    {"romance",                  3600}, {"russian",                 10800},
-    {"sa eastern",             -10800}, {"sa pacific",             -18000},
-    {"sa western",             -14400}, {"samoa",                  -39600},
-    {"se asia",                 25200}, {"malay peninsula",         28800},
-    {"south africa",             7200}, {"sri lanka",               21600},
-    {"taipei",                  28800}, {"tasmania",                36000},
-    {"tokyo",                   32400}, {"tonga",                   46800},
-    {"us eastern",             -18000}, {"us mountain",            -25200},
-    {"vladivostok",             36000}, {"w. australia",            28800},
-    {"w. central africa",        3600}, {"w. europe",                3600},
-    {"west asia",               18000}, {"west pacific",            36000},
-    {"yakutsk",                 32400}
-};
+#include "zonetab.h"
 
 VALUE
 date_zone_to_diff(VALUE str)
 {
     VALUE offset = Qnil;
+    VALUE vbuf = 0;
 
     long l, i;
     char *s, *dest, *d;
@@ -411,7 +352,7 @@ date_zone_to_diff(VALUE str)
     l = RSTRING_LEN(str);
     s = RSTRING_PTR(str);
 
-    dest = d = ALLOCA_N(char, l + 1);
+    dest = d = ALLOCV_N(char, vbuf, l + 1);
 
     for (i = 0; i < l; i++) {
 	if (isspace((unsigned char)s[i]) || s[i] == '\0') {
@@ -469,24 +410,12 @@ date_zone_to_diff(VALUE str)
 #undef DST
 	}
 	{
-	    static VALUE zones = Qnil;
-
-	    if (NIL_P(zones)) {
-		int i;
-
-		zones = rb_hash_new();
-		rb_gc_register_mark_object(zones);
-		for (i = 0; i < (int)sizeof_array(zones_source); i++) {
-		    VALUE name = rb_str_new2(zones_source[i].name);
-		    VALUE offset = INT2FIX(zones_source[i].offset);
-		    rb_hash_aset(zones, name, offset);
-		}
-	    }
-
-	    offset = f_aref(zones, str);
-	    if (!NIL_P(offset)) {
+	    const struct zone *z = zonetab(RSTRING_PTR(str), (unsigned int)RSTRING_LEN(str));
+	    if (z) {
+		int d = z->offset;
 		if (dst)
-		    offset = f_add(offset, INT2FIX(3600));
+		    d += 3600;
+		offset = INT2FIX(d);
 		goto ok;
 	    }
 	}
@@ -508,13 +437,13 @@ date_zone_to_diff(VALUE str)
 
 		str = rb_str_new2(s);
 
-		if (p = strchr(s, ':')) {
+		if ((p = strchr(s, ':')) != NULL) {
 		    hour = rb_str_new(s, p - s);
 		    s = ++p;
-		    if (p = strchr(s, ':')) {
+		    if ((p = strchr(s, ':')) != NULL) {
 			min = rb_str_new(s, p - s);
 			s = ++p;
-			if (p = strchr(s, ':')) {
+			if ((p = strchr(s, ':')) != NULL) {
 			    sec = rb_str_new(s, p - s);
 			}
 			else
@@ -526,9 +455,10 @@ date_zone_to_diff(VALUE str)
 		    goto num;
 		}
 		if (strpbrk(RSTRING_PTR(str), ",.")) {
+		    VALUE astr = 0;
 		    char *a, *b;
 
-		    a = ALLOCA_N(char, RSTRING_LEN(str) + 1);
+		    a = ALLOCV_N(char, astr, RSTRING_LEN(str) + 1);
 		    strcpy(a, RSTRING_PTR(str));
 		    b = strpbrk(a, ",.");
 		    *b = '\0';
@@ -540,6 +470,7 @@ date_zone_to_diff(VALUE str)
 				 f_expt(INT2FIX(10),
 					LONG2NUM((long)strlen(b)))),
 				INT2FIX(60));
+		    ALLOCV_END(astr);
 		    goto num;
 		}
 		{
@@ -552,7 +483,7 @@ date_zone_to_diff(VALUE str)
 			if (cl >= 3)
 			    min  = rb_str_new(&cs[1], 2);
 			if (cl >= 5)
-			    min  = rb_str_new(&cs[3], 2);
+			    sec  = rb_str_new(&cs[3], 2);
 		    }
 		    else {
 			if (cl >= 2)
@@ -568,12 +499,12 @@ date_zone_to_diff(VALUE str)
 		if (NIL_P(hour))
 		    offset = INT2FIX(0);
 		else {
-		    if (TYPE(hour) == T_STRING)
+		    if (RB_TYPE_P(hour, T_STRING))
 			hour = str2num(hour);
 		    offset = f_mul(hour, INT2FIX(3600));
 		}
 		if (!NIL_P(min)) {
-		    if (TYPE(min) == T_STRING)
+		    if (RB_TYPE_P(min, T_STRING))
 			min = str2num(min);
 		    offset = f_add(offset, f_mul(min, INT2FIX(60)));
 		}
@@ -588,6 +519,7 @@ date_zone_to_diff(VALUE str)
     }
     RB_GC_GUARD(str);
   ok:
+    ALLOCV_END(vbuf);
     return offset;
 }
 
@@ -748,9 +680,9 @@ parse_time(VALUE str, VALUE hash)
 		   "("
 		     "(?:gmt|utc?)?[-+]\\d+(?:[,.:]\\d+(?::\\d+)?)?"
 		   "|"
-		     "[[:alpha:].\\s]+(?:standard|daylight)\\stime\\b"
+		     "(?-i:[[:alpha:].\\s]+)(?:standard|daylight)\\stime\\b"
 		   "|"
-		     "[[:alpha:]]+(?:\\sdst)?\\b"
+		     "(?-i:[[:alpha:]]+)(?:\\sdst)?\\b"
 		   ")"
 		")?";
     static VALUE pat = Qnil;
@@ -763,6 +695,104 @@ parse_time(VALUE str, VALUE hash)
 #endif
 }
 
+#ifdef TIGHT_PARSER
+static int
+parse_era1_cb(VALUE m, VALUE hash)
+{
+    return 1;
+}
+
+static int
+parse_era1(VALUE str, VALUE hash)
+{
+    static const char pat_source[] =
+	"(a(?:d|\\.d\\.))";
+    static VALUE pat = Qnil;
+
+    REGCOMP_I(pat);
+    SUBA(str, pat, parse_era1_cb);
+}
+
+static int
+parse_era2_cb(VALUE m, VALUE hash)
+{
+    VALUE b;
+
+    b = rb_reg_nth_match(1, m);
+    if (*RSTRING_PTR(b) == 'B' ||
+	*RSTRING_PTR(b) == 'b')
+	set_hash("_bc", Qtrue);
+    return 1;
+}
+
+static int
+parse_era2(VALUE str, VALUE hash)
+{
+    static const char pat_source[] =
+	"(c(?:e|\\.e\\.)|b(?:ce|\\.c\\.e\\.)|b(?:c|\\.c\\.))";
+    static VALUE pat = Qnil;
+
+    REGCOMP_I(pat);
+    SUBB(str, pat, parse_era2_cb);
+}
+
+static int
+parse_era(VALUE str, VALUE hash)
+{
+    if (parse_era1(str, hash)) /* pre */
+	goto ok;
+    if (parse_era2(str, hash)) /* post */
+	goto ok;
+    return 0;
+  ok:
+    return 1;
+}
+#endif
+
+#ifdef TIGHT_PARSER
+static int
+check_year_width(VALUE y)
+{
+    char *s;
+    size_t l;
+
+    s = RSTRING_PTR(y);
+    l = strcspn(s, DECDIGIT);
+    s += l;
+    l = strspn(s, DECDIGIT);
+    if (l != 2)
+	return 0;
+    return 1;
+}
+
+static int
+check_apost(VALUE a, VALUE b, VALUE c)
+{
+    int f = 0;
+
+    if (!NIL_P(a) && *RSTRING_PTR(a) == '\'') {
+	if (!check_year_width(a))
+	    return 0;
+	f++;
+    }
+    if (!NIL_P(b) && *RSTRING_PTR(b) == '\'') {
+	if (!check_year_width(b))
+	    return 0;
+	if (!NIL_P(c))
+	    return 0;
+	f++;
+    }
+    if (!NIL_P(c) && *RSTRING_PTR(c) == '\'') {
+	if (!check_year_width(c))
+	    return 0;
+	f++;
+    }
+    if (f > 1)
+	return 0;
+    return 1;
+}
+#endif
+
 static int
 parse_eu_cb(VALUE m, VALUE hash)
 {
@@ -786,6 +816,9 @@ parse_eu_cb(VALUE m, VALUE hash)
     mon = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
+    if (!check_apost(d, mon, y))
+	return 0;
+
     mon = INT2FIX(mon_num(mon));
 
     s3e(hash, y, mon, d, 0);
@@ -802,9 +835,9 @@ parse_eu(VALUE str, VALUE hash)
 		FPW_COM FPT_COM
 #endif
 #ifndef TIGHT_PARSER
-		"'?(\\d+)[^-\\d\\s]*"
+		"('?\\d+)[^-\\d\\s]*"
 #else
-		"(\\d+)(?:st|nd|rd|th)?\\b"
+		"(\\d+)(?:(?:st|nd|rd|th)\\b)?"
 #endif
 		 "\\s*"
 #ifndef TIGHT_PARSER
@@ -819,7 +852,11 @@ parse_eu(VALUE str, VALUE hash)
 		   "\\s*"
 		   "('?-?\\d+(?:(?:st|nd|rd|th)\\b)?)"
 #else
-		   "('?-?\\d+)"
+		   "(?:" FPA ")?"
+		   "\\s*"
+		   "([-']?\\d+)"
+		   "\\s*"
+		   "(?:" FPA "|" FPB ")?"
 #endif
 		")?"
 #ifdef TIGHT_PARSER
@@ -857,6 +894,9 @@ parse_us_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
+    if (!check_apost(mon, d, y))
+	return 0;
+
     mon = INT2FIX(mon_num(mon));
 
     s3e(hash, y, mon, d, 0);
@@ -881,7 +921,7 @@ parse_us(VALUE str, VALUE hash)
 #ifndef TIGHT_PARSER
 		 "('?\\d+)[^-\\d\\s']*"
 #else
-		 "(\\d+)(?:st|nd|rd|th)?\\b"
+		 "('?\\d+)(?:(?:st|nd|rd|th)\\b)?"
 		COM_FPT
 #endif
 		 "(?:"
@@ -890,8 +930,14 @@ parse_us(VALUE str, VALUE hash)
 #ifndef TIGHT_PARSER
 		   "(c(?:e|\\.e\\.)|b(?:ce|\\.c\\.e\\.)|a(?:d|\\.d\\.)|b(?:c|\\.c\\.))?"
 		   "\\s*"
-#endif
 		   "('?-?\\d+)"
+#else
+		   "(?:" FPA ")?"
+		   "\\s*"
+		   "([-']?\\d+)"
+		   "\\s*"
+		   "(?:" FPA "|" FPB ")?"
+#endif
 		")?"
 #ifdef TIGHT_PARSER
 		COM_FPT COM_FPW
@@ -914,7 +960,7 @@ parse_iso_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(3, m);
 
 #ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(y) == '\'' && *RSTRING_PTR(d) == '\'')
+    if (!check_apost(y, mon, d))
 	return 0;
 #endif
 
@@ -931,7 +977,7 @@ parse_iso(VALUE str, VALUE hash)
 #else
 	BOS
 	FPW_COM FPT_COM
-	"('?[-+]?\\d+)-(\\d+)-('?-?\\d+)"
+	"([-+']?\\d+)-(\\d+)-([-']?\\d+)"
 	TEE_FPT COM_FPW
 	EOS
 #endif
@@ -1262,7 +1308,7 @@ parse_vms11_cb(VALUE m, VALUE hash)
     y = rb_reg_nth_match(3, m);
 
 #ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y) && *RSTRING_PTR(y) == '\'')
+    if (!check_apost(d, mon, y))
 	return 0;
 #endif
 
@@ -1282,8 +1328,8 @@ parse_vms11(VALUE str, VALUE hash)
 #else
 	BOS
 	FPW_COM FPT_COM
-	"('?-?\\d+)-(" DOTLESS_VALID_MONTHS ")"
-	"-('?-?\\d+)"
+	"([-']?\\d+)-(" DOTLESS_VALID_MONTHS ")"
+	"-([-']?\\d+)"
 	COM_FPT COM_FPW
 	EOS
 #endif
@@ -1304,7 +1350,7 @@ parse_vms12_cb(VALUE m, VALUE hash)
     y = rb_reg_nth_match(3, m);
 
 #ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y))
+    if (!check_apost(mon, d, y))
 	return 0;
 #endif
 
@@ -1325,7 +1371,7 @@ parse_vms12(VALUE str, VALUE hash)
 	BOS
 	FPW_COM FPT_COM
 	"(" DOTLESS_VALID_MONTHS ")"
-	"-('?-?\\d+)(?:-('?-?\\d+))?"
+	"-([-']?\\d+)(?:-([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 #endif
@@ -1359,11 +1405,7 @@ parse_sla_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(3, m);
 
 #ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(y) == '\'' &&
-	(*RSTRING_PTR(mon) == '\'' ||
-	 !NIL_P(d) && *RSTRING_PTR(d) == '\''))
-	return 0;
-    if (*RSTRING_PTR(mon) == '\'' && !NIL_P(d))
+    if (!check_apost(y, mon, d))
 	return 0;
 #endif
 
@@ -1380,7 +1422,7 @@ parse_sla(VALUE str, VALUE hash)
 #else
 	BOS
 	FPW_COM FPT_COM
-	"('?-?\\d+)/\\s*('?\\d+)(?:(?:[-/]|\\s+)\\s*('?-?\\d+))?"
+	"([-']?\\d+)/\\s*('?\\d+)(?:(?:[-/]|\\s+)\\s*([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 #endif
@@ -1401,10 +1443,8 @@ parse_sla2_cb(VALUE m, VALUE hash)
     mon = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
-#ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y) && *RSTRING_PTR(y) == '\'')
+    if (!check_apost(d, mon, y))
 	return 0;
-#endif
 
     mon = INT2FIX(mon_num(mon));
 
@@ -1418,7 +1458,7 @@ parse_sla2(VALUE str, VALUE hash)
     static const char pat_source[] =
 	BOS
 	FPW_COM FPT_COM
-	"('?-?\\d+)/\\s*(" DOTLESS_VALID_MONTHS ")(?:(?:[-/]|\\s+)\\s*('?-?\\d+))?"
+	"([-']?\\d+)/\\s*(" DOTLESS_VALID_MONTHS ")(?:(?:[-/]|\\s+)\\s*([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 	;
@@ -1437,10 +1477,8 @@ parse_sla3_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
-#ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y))
+    if (!check_apost(mon, d, y))
 	return 0;
-#endif
 
     mon = INT2FIX(mon_num(mon));
 
@@ -1454,7 +1492,7 @@ parse_sla3(VALUE str, VALUE hash)
     static const char pat_source[] =
 	BOS
 	FPW_COM FPT_COM
-	"(" DOTLESS_VALID_MONTHS ")/\\s*('?\\d+)(?:(?:[-/]|\\s+)\\s*('?-?\\d+))?"
+	"(" DOTLESS_VALID_MONTHS ")/\\s*([-']?\\d+)(?:(?:[-/]|\\s+)\\s*([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 	;
@@ -1475,7 +1513,7 @@ parse_dot_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(3, m);
 
 #ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(y) == '\'' && *RSTRING_PTR(d) == '\'')
+    if (!check_apost(y, mon, d))
 	return 0;
 #endif
 
@@ -1492,7 +1530,7 @@ parse_dot(VALUE str, VALUE hash)
 #else
 	BOS
 	FPW_COM FPT_COM
-	"('?-?\\d+)\\.\\s*(\\d+)\\.\\s*('?-?\\d+)"
+	"([-']?\\d+)\\.\\s*(\\d+)\\.\\s*([-']?\\d+)"
 	COM_FPT COM_FPW
 	EOS
 #endif
@@ -1513,10 +1551,8 @@ parse_dot2_cb(VALUE m, VALUE hash)
     mon = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
-#ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y) && *RSTRING_PTR(y) == '\'')
+    if (!check_apost(d, mon, y))
 	return 0;
-#endif
 
     mon = INT2FIX(mon_num(mon));
 
@@ -1530,7 +1566,7 @@ parse_dot2(VALUE str, VALUE hash)
     static const char pat_source[] =
 	BOS
 	FPW_COM FPT_COM
-	"('?-?\\d+)\\.\\s*(" DOTLESS_VALID_MONTHS ")(?:(?:[./])\\s*('?-?\\d+))?"
+	"([-']?\\d+)\\.\\s*(" DOTLESS_VALID_MONTHS ")(?:(?:[./])\\s*([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 	;
@@ -1549,10 +1585,8 @@ parse_dot3_cb(VALUE m, VALUE hash)
     d = rb_reg_nth_match(2, m);
     y = rb_reg_nth_match(3, m);
 
-#ifdef TIGHT_PARSER
-    if (*RSTRING_PTR(d) == '\'' && !NIL_P(y))
+    if (!check_apost(mon, d, y))
 	return 0;
-#endif
 
     mon = INT2FIX(mon_num(mon));
 
@@ -1566,7 +1600,7 @@ parse_dot3(VALUE str, VALUE hash)
     static const char pat_source[] =
 	BOS
 	FPW_COM FPT_COM
-	"(" DOTLESS_VALID_MONTHS ")\\.\\s*('?\\d+)(?:(?:[./])\\s*('?-?\\d+))?"
+	"(" DOTLESS_VALID_MONTHS ")\\.\\s*([-']?\\d+)(?:(?:[./])\\s*([-']?\\d+))?"
 	COM_FPT COM_FPW
 	EOS
 	;
@@ -1856,7 +1890,8 @@ parse_ddd_cb(VALUE m, VALUE hash)
 	set_hash("zone", s5);
 
 	if (*cs5 == '[') {
-	    char *buf = ALLOCA_N(char, l5 + 1);
+	    VALUE vbuf = 0;
+	    char *buf = ALLOCV_N(char, vbuf, l5 + 1);
 	    char *s1, *s2, *s3;
 	    VALUE zone;
 
@@ -1878,6 +1913,7 @@ parse_ddd_cb(VALUE m, VALUE hash)
 	    if (isdigit((unsigned char)*s1))
 		*--s1 = '+';
 	    set_hash("offset", date_zone_to_diff(rb_str_new2(s1)));
+	    ALLOCV_END(vbuf);
 	}
 	RB_GC_GUARD(s5);
     }
@@ -1923,12 +1959,7 @@ parse_ddd(VALUE str, VALUE hash)
 static int
 parse_bc_cb(VALUE m, VALUE hash)
 {
-    VALUE y;
-
-    y = ref_hash("year");
-    if (!NIL_P(y))
-	set_hash("year", f_add(f_negate(y), INT2FIX(1)));
-
+    set_hash("_bc", Qtrue);
     return 1;
 }
 
@@ -2097,12 +2128,17 @@ date__parse(VALUE str, VALUE comp)
     if (HAVE_ELEM_P(HAVE_DIGIT))
 	parse_time(str, hash);
 
-    if (HAVE_ELEM_P(HAVE_ALPHA|HAVE_DIGIT))
+#ifdef TIGHT_PARSER
+    if (HAVE_ELEM_P(HAVE_ALPHA))
+	parse_era(str, hash);
+#endif
+
+    if (HAVE_ELEM_P(HAVE_ALPHA|HAVE_DIGIT)) {
 	if (parse_eu(str, hash))
 	    goto ok;
-    if (HAVE_ELEM_P(HAVE_ALPHA|HAVE_DIGIT))
 	if (parse_us(str, hash))
 	    goto ok;
+    }
     if (HAVE_ELEM_P(HAVE_DIGIT|HAVE_DASH))
 	if (parse_iso(str, hash))
 	    goto ok;
@@ -2170,6 +2206,21 @@ date__parse(VALUE str, VALUE comp)
 #endif
 
     {
+	if (RTEST(ref_hash("_bc"))) {
+	    VALUE y;
+
+	    y = ref_hash("cwyear");
+	    if (!NIL_P(y)) {
+		y = f_add(f_negate(y), INT2FIX(1));
+		set_hash("cwyear", y);
+	    }
+	    y = ref_hash("year");
+	    if (!NIL_P(y)) {
+		y = f_add(f_negate(y), INT2FIX(1));
+		set_hash("year", y);
+	    }
+	}
+
 	if (RTEST(ref_hash("_comp"))) {
 	    VALUE y;
 
@@ -2190,8 +2241,10 @@ date__parse(VALUE str, VALUE comp)
 			set_hash("year", f_add(y, INT2FIX(2000)));
 		}
 	}
+
     }
 
+    del_hash("_bc");
     del_hash("_comp");
 
     {
@@ -2712,7 +2765,9 @@ rfc2822_cb(VALUE m, VALUE hash)
 	    s[i] = rb_reg_nth_match(i, m);
     }
 
-    set_hash("wday", INT2FIX(day_num(s[1])));
+    if (!NIL_P(s[1])) {
+	set_hash("wday", INT2FIX(day_num(s[1])));
+    }
     set_hash("mday", str2num(s[2]));
     set_hash("mon", INT2FIX(mon_num(s[3])));
     y = str2num(s[4]);
