@@ -1,5 +1,5 @@
 /*
-    $Id: strscan.c 50511 2015-05-16 12:56:48Z nobu $
+    $Id: strscan.c 53715 2016-02-02 04:39:44Z naruse $
 
     Copyright (c) 1999-2006 Minero Aoki
 
@@ -181,11 +181,7 @@ static size_t
 strscan_memsize(const void *ptr)
 {
     const struct strscanner *p = ptr;
-    size_t size = 0;
-    if (p) {
-	size = sizeof(*p) - sizeof(p->regs) + onig_region_memsize(&p->regs);
-    }
-    return size;
+    return sizeof(*p) - sizeof(p->regs) + onig_region_memsize(&p->regs);
 }
 
 static const rb_data_type_t strscanner_type = {
@@ -464,7 +460,7 @@ strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
 
     p->regex = regex;
     re = rb_reg_prepare_re(regex, p->str);
-    tmpreg = re != RREGEXP(regex)->ptr;
+    tmpreg = re != RREGEXP_PTR(regex);
     if (!tmpreg) RREGEXP(regex)->usecnt++;
 
     if (headonly) {
@@ -484,8 +480,8 @@ strscan_do_scan(VALUE self, VALUE regex, int succptr, int getstr, int headonly)
             onig_free(re);
         }
         else {
-            onig_free(RREGEXP(regex)->ptr);
-            RREGEXP(regex)->ptr = re;
+            onig_free(RREGEXP_PTR(regex));
+            RREGEXP_PTR(regex) = re;
         }
     }
 
@@ -982,7 +978,7 @@ name_to_backref_number(struct re_registers *regs, VALUE regexp, const char* name
 {
     int num;
 
-    num = onig_name_to_backref_number(RREGEXP(regexp)->ptr,
+    num = onig_name_to_backref_number(RREGEXP_PTR(regexp),
 	(const unsigned char* )name, (const unsigned char* )name_end, regs);
     if (num >= 1) {
 	return num;
@@ -1340,7 +1336,7 @@ Init_strscan(void)
     tmp = rb_str_new2(STRSCAN_VERSION);
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Version"), tmp);
-    tmp = rb_str_new2("$Id: strscan.c 50511 2015-05-16 12:56:48Z nobu $");
+    tmp = rb_str_new2("$Id: strscan.c 53715 2016-02-02 04:39:44Z naruse $");
     rb_obj_freeze(tmp);
     rb_const_set(StringScanner, rb_intern("Id"), tmp);
 
